'''
Autogenerate code from xml spec
'''


ExtensionClasses = {
    ObjectIds.TrustListDataType_Encoding_DefaultBinary: TrustListDataType,
    ObjectIds.Argument_Encoding_DefaultBinary: Argument,
    ObjectIds.EnumValueType_Encoding_DefaultBinary: EnumValueType,
    ObjectIds.OptionSet_Encoding_DefaultBinary: OptionSet,
    ObjectIds.Union_Encoding_DefaultBinary: Union,
    ObjectIds.TimeZoneDataType_Encoding_DefaultBinary: TimeZoneDataType,
    ObjectIds.ApplicationDescription_Encoding_DefaultBinary: ApplicationDescription,
    ObjectIds.RequestHeader_Encoding_DefaultBinary: RequestHeader,
    ObjectIds.ResponseHeader_Encoding_DefaultBinary: ResponseHeader,
    ObjectIds.ServiceFault_Encoding_DefaultBinary: ServiceFault,
    ObjectIds.FindServersRequest_Encoding_DefaultBinary: FindServersRequest,
    ObjectIds.FindServersResponse_Encoding_DefaultBinary: FindServersResponse,
    ObjectIds.ServerOnNetwork_Encoding_DefaultBinary: ServerOnNetwork,
    ObjectIds.FindServersOnNetworkRequest_Encoding_DefaultBinary: FindServersOnNetworkRequest,
    ObjectIds.FindServersOnNetworkResponse_Encoding_DefaultBinary: FindServersOnNetworkResponse,
    ObjectIds.UserTokenPolicy_Encoding_DefaultBinary: UserTokenPolicy,
    ObjectIds.EndpointDescription_Encoding_DefaultBinary: EndpointDescription,
    ObjectIds.GetEndpointsRequest_Encoding_DefaultBinary: GetEndpointsRequest,
    ObjectIds.GetEndpointsResponse_Encoding_DefaultBinary: GetEndpointsResponse,
    ObjectIds.RegisteredServer_Encoding_DefaultBinary: RegisteredServer,
    ObjectIds.RegisterServerRequest_Encoding_DefaultBinary: RegisterServerRequest,
    ObjectIds.RegisterServerResponse_Encoding_DefaultBinary: RegisterServerResponse,
    ObjectIds.DiscoveryConfiguration_Encoding_DefaultBinary: DiscoveryConfiguration,
    ObjectIds.MdnsDiscoveryConfiguration_Encoding_DefaultBinary: MdnsDiscoveryConfiguration,
    ObjectIds.RegisterServer2Request_Encoding_DefaultBinary: RegisterServer2Request,
    ObjectIds.RegisterServer2Response_Encoding_DefaultBinary: RegisterServer2Response,
    ObjectIds.ChannelSecurityToken_Encoding_DefaultBinary: ChannelSecurityToken,
    ObjectIds.OpenSecureChannelRequest_Encoding_DefaultBinary: OpenSecureChannelRequest,
    ObjectIds.OpenSecureChannelResponse_Encoding_DefaultBinary: OpenSecureChannelResponse,
    ObjectIds.CloseSecureChannelRequest_Encoding_DefaultBinary: CloseSecureChannelRequest,
    ObjectIds.CloseSecureChannelResponse_Encoding_DefaultBinary: CloseSecureChannelResponse,
    ObjectIds.SignedSoftwareCertificate_Encoding_DefaultBinary: SignedSoftwareCertificate,
    ObjectIds.SignatureData_Encoding_DefaultBinary: SignatureData,
    ObjectIds.CreateSessionRequest_Encoding_DefaultBinary: CreateSessionRequest,
    ObjectIds.CreateSessionResponse_Encoding_DefaultBinary: CreateSessionResponse,
    ObjectIds.UserIdentityToken_Encoding_DefaultBinary: UserIdentityToken,
    ObjectIds.AnonymousIdentityToken_Encoding_DefaultBinary: AnonymousIdentityToken,
    ObjectIds.UserNameIdentityToken_Encoding_DefaultBinary: UserNameIdentityToken,
    ObjectIds.X509IdentityToken_Encoding_DefaultBinary: X509IdentityToken,
    ObjectIds.KerberosIdentityToken_Encoding_DefaultBinary: KerberosIdentityToken,
    ObjectIds.IssuedIdentityToken_Encoding_DefaultBinary: IssuedIdentityToken,
    ObjectIds.ActivateSessionRequest_Encoding_DefaultBinary: ActivateSessionRequest,
    ObjectIds.ActivateSessionResponse_Encoding_DefaultBinary: ActivateSessionResponse,
    ObjectIds.CloseSessionRequest_Encoding_DefaultBinary: CloseSessionRequest,
    ObjectIds.CloseSessionResponse_Encoding_DefaultBinary: CloseSessionResponse,
    ObjectIds.CancelRequest_Encoding_DefaultBinary: CancelRequest,
    ObjectIds.CancelResponse_Encoding_DefaultBinary: CancelResponse,
    ObjectIds.NodeAttributes_Encoding_DefaultBinary: NodeAttributes,
    ObjectIds.ObjectAttributes_Encoding_DefaultBinary: ObjectAttributes,
    ObjectIds.VariableAttributes_Encoding_DefaultBinary: VariableAttributes,
    ObjectIds.MethodAttributes_Encoding_DefaultBinary: MethodAttributes,
    ObjectIds.ObjectTypeAttributes_Encoding_DefaultBinary: ObjectTypeAttributes,
    ObjectIds.VariableTypeAttributes_Encoding_DefaultBinary: VariableTypeAttributes,
    ObjectIds.ReferenceTypeAttributes_Encoding_DefaultBinary: ReferenceTypeAttributes,
    ObjectIds.DataTypeAttributes_Encoding_DefaultBinary: DataTypeAttributes,
    ObjectIds.ViewAttributes_Encoding_DefaultBinary: ViewAttributes,
    ObjectIds.AddNodesItem_Encoding_DefaultBinary: AddNodesItem,
    ObjectIds.AddNodesResult_Encoding_DefaultBinary: AddNodesResult,
    ObjectIds.AddNodesRequest_Encoding_DefaultBinary: AddNodesRequest,
    ObjectIds.AddNodesResponse_Encoding_DefaultBinary: AddNodesResponse,
    ObjectIds.AddReferencesItem_Encoding_DefaultBinary: AddReferencesItem,
    ObjectIds.AddReferencesRequest_Encoding_DefaultBinary: AddReferencesRequest,
    ObjectIds.AddReferencesResponse_Encoding_DefaultBinary: AddReferencesResponse,
    ObjectIds.DeleteNodesItem_Encoding_DefaultBinary: DeleteNodesItem,
    ObjectIds.DeleteNodesRequest_Encoding_DefaultBinary: DeleteNodesRequest,
    ObjectIds.DeleteNodesResponse_Encoding_DefaultBinary: DeleteNodesResponse,
    ObjectIds.DeleteReferencesItem_Encoding_DefaultBinary: DeleteReferencesItem,
    ObjectIds.DeleteReferencesRequest_Encoding_DefaultBinary: DeleteReferencesRequest,
    ObjectIds.DeleteReferencesResponse_Encoding_DefaultBinary: DeleteReferencesResponse,
    ObjectIds.ViewDescription_Encoding_DefaultBinary: ViewDescription,
    ObjectIds.BrowseDescription_Encoding_DefaultBinary: BrowseDescription,
    ObjectIds.ReferenceDescription_Encoding_DefaultBinary: ReferenceDescription,
    ObjectIds.BrowseResult_Encoding_DefaultBinary: BrowseResult,
    ObjectIds.BrowseRequest_Encoding_DefaultBinary: BrowseRequest,
    ObjectIds.BrowseResponse_Encoding_DefaultBinary: BrowseResponse,
    ObjectIds.BrowseNextRequest_Encoding_DefaultBinary: BrowseNextRequest,
    ObjectIds.BrowseNextResponse_Encoding_DefaultBinary: BrowseNextResponse,
    ObjectIds.RelativePathElement_Encoding_DefaultBinary: RelativePathElement,
    ObjectIds.RelativePath_Encoding_DefaultBinary: RelativePath,
    ObjectIds.BrowsePath_Encoding_DefaultBinary: BrowsePath,
    ObjectIds.BrowsePathTarget_Encoding_DefaultBinary: BrowsePathTarget,
    ObjectIds.BrowsePathResult_Encoding_DefaultBinary: BrowsePathResult,
    ObjectIds.TranslateBrowsePathsToNodeIdsRequest_Encoding_DefaultBinary: TranslateBrowsePathsToNodeIdsRequest,
    ObjectIds.TranslateBrowsePathsToNodeIdsResponse_Encoding_DefaultBinary: TranslateBrowsePathsToNodeIdsResponse,
    ObjectIds.RegisterNodesRequest_Encoding_DefaultBinary: RegisterNodesRequest,
    ObjectIds.RegisterNodesResponse_Encoding_DefaultBinary: RegisterNodesResponse,
    ObjectIds.UnregisterNodesRequest_Encoding_DefaultBinary: UnregisterNodesRequest,
    ObjectIds.UnregisterNodesResponse_Encoding_DefaultBinary: UnregisterNodesResponse,
    ObjectIds.EndpointConfiguration_Encoding_DefaultBinary: EndpointConfiguration,
    ObjectIds.SupportedProfile_Encoding_DefaultBinary: SupportedProfile,
    ObjectIds.SoftwareCertificate_Encoding_DefaultBinary: SoftwareCertificate,
    ObjectIds.QueryDataDescription_Encoding_DefaultBinary: QueryDataDescription,
    ObjectIds.NodeTypeDescription_Encoding_DefaultBinary: NodeTypeDescription,
    ObjectIds.QueryDataSet_Encoding_DefaultBinary: QueryDataSet,
    ObjectIds.NodeReference_Encoding_DefaultBinary: NodeReference,
    ObjectIds.ContentFilterElement_Encoding_DefaultBinary: ContentFilterElement,
    ObjectIds.ContentFilter_Encoding_DefaultBinary: ContentFilter,
    ObjectIds.ElementOperand_Encoding_DefaultBinary: ElementOperand,
    ObjectIds.LiteralOperand_Encoding_DefaultBinary: LiteralOperand,
    ObjectIds.AttributeOperand_Encoding_DefaultBinary: AttributeOperand,
    ObjectIds.SimpleAttributeOperand_Encoding_DefaultBinary: SimpleAttributeOperand,
    ObjectIds.ContentFilterElementResult_Encoding_DefaultBinary: ContentFilterElementResult,
    ObjectIds.ContentFilterResult_Encoding_DefaultBinary: ContentFilterResult,
    ObjectIds.ParsingResult_Encoding_DefaultBinary: ParsingResult,
    ObjectIds.QueryFirstRequest_Encoding_DefaultBinary: QueryFirstRequest,
    ObjectIds.QueryFirstResponse_Encoding_DefaultBinary: QueryFirstResponse,
    ObjectIds.QueryNextRequest_Encoding_DefaultBinary: QueryNextRequest,
    ObjectIds.QueryNextResponse_Encoding_DefaultBinary: QueryNextResponse,
    ObjectIds.ReadValueId_Encoding_DefaultBinary: ReadValueId,
    ObjectIds.ReadRequest_Encoding_DefaultBinary: ReadRequest,
    ObjectIds.ReadResponse_Encoding_DefaultBinary: ReadResponse,
    ObjectIds.HistoryReadValueId_Encoding_DefaultBinary: HistoryReadValueId,
    ObjectIds.HistoryReadResult_Encoding_DefaultBinary: HistoryReadResult,
    ObjectIds.HistoryReadDetails_Encoding_DefaultBinary: HistoryReadDetails,
    ObjectIds.ReadEventDetails_Encoding_DefaultBinary: ReadEventDetails,
    ObjectIds.ReadRawModifiedDetails_Encoding_DefaultBinary: ReadRawModifiedDetails,
    ObjectIds.ReadProcessedDetails_Encoding_DefaultBinary: ReadProcessedDetails,
    ObjectIds.ReadAtTimeDetails_Encoding_DefaultBinary: ReadAtTimeDetails,
    ObjectIds.HistoryData_Encoding_DefaultBinary: HistoryData,
    ObjectIds.ModificationInfo_Encoding_DefaultBinary: ModificationInfo,
    ObjectIds.HistoryModifiedData_Encoding_DefaultBinary: HistoryModifiedData,
    ObjectIds.HistoryEvent_Encoding_DefaultBinary: HistoryEvent,
    ObjectIds.HistoryReadRequest_Encoding_DefaultBinary: HistoryReadRequest,
    ObjectIds.HistoryReadResponse_Encoding_DefaultBinary: HistoryReadResponse,
    ObjectIds.WriteValue_Encoding_DefaultBinary: WriteValue,
    ObjectIds.WriteRequest_Encoding_DefaultBinary: WriteRequest,
    ObjectIds.WriteResponse_Encoding_DefaultBinary: WriteResponse,
    ObjectIds.HistoryUpdateDetails_Encoding_DefaultBinary: HistoryUpdateDetails,
    ObjectIds.UpdateDataDetails_Encoding_DefaultBinary: UpdateDataDetails,
    ObjectIds.UpdateStructureDataDetails_Encoding_DefaultBinary: UpdateStructureDataDetails,
    ObjectIds.UpdateEventDetails_Encoding_DefaultBinary: UpdateEventDetails,
    ObjectIds.DeleteRawModifiedDetails_Encoding_DefaultBinary: DeleteRawModifiedDetails,
    ObjectIds.DeleteAtTimeDetails_Encoding_DefaultBinary: DeleteAtTimeDetails,
    ObjectIds.DeleteEventDetails_Encoding_DefaultBinary: DeleteEventDetails,
    ObjectIds.HistoryUpdateResult_Encoding_DefaultBinary: HistoryUpdateResult,
    ObjectIds.HistoryUpdateRequest_Encoding_DefaultBinary: HistoryUpdateRequest,
    ObjectIds.HistoryUpdateResponse_Encoding_DefaultBinary: HistoryUpdateResponse,
    ObjectIds.CallMethodRequest_Encoding_DefaultBinary: CallMethodRequest,
    ObjectIds.CallMethodResult_Encoding_DefaultBinary: CallMethodResult,
    ObjectIds.CallRequest_Encoding_DefaultBinary: CallRequest,
    ObjectIds.CallResponse_Encoding_DefaultBinary: CallResponse,
    ObjectIds.MonitoringFilter_Encoding_DefaultBinary: MonitoringFilter,
    ObjectIds.DataChangeFilter_Encoding_DefaultBinary: DataChangeFilter,
    ObjectIds.EventFilter_Encoding_DefaultBinary: EventFilter,
    ObjectIds.AggregateConfiguration_Encoding_DefaultBinary: AggregateConfiguration,
    ObjectIds.AggregateFilter_Encoding_DefaultBinary: AggregateFilter,
    ObjectIds.MonitoringFilterResult_Encoding_DefaultBinary: MonitoringFilterResult,
    ObjectIds.EventFilterResult_Encoding_DefaultBinary: EventFilterResult,
    ObjectIds.AggregateFilterResult_Encoding_DefaultBinary: AggregateFilterResult,
    ObjectIds.MonitoringParameters_Encoding_DefaultBinary: MonitoringParameters,
    ObjectIds.MonitoredItemCreateRequest_Encoding_DefaultBinary: MonitoredItemCreateRequest,
    ObjectIds.MonitoredItemCreateResult_Encoding_DefaultBinary: MonitoredItemCreateResult,
    ObjectIds.CreateMonitoredItemsRequest_Encoding_DefaultBinary: CreateMonitoredItemsRequest,
    ObjectIds.CreateMonitoredItemsResponse_Encoding_DefaultBinary: CreateMonitoredItemsResponse,
    ObjectIds.MonitoredItemModifyRequest_Encoding_DefaultBinary: MonitoredItemModifyRequest,
    ObjectIds.MonitoredItemModifyResult_Encoding_DefaultBinary: MonitoredItemModifyResult,
    ObjectIds.ModifyMonitoredItemsRequest_Encoding_DefaultBinary: ModifyMonitoredItemsRequest,
    ObjectIds.ModifyMonitoredItemsResponse_Encoding_DefaultBinary: ModifyMonitoredItemsResponse,
    ObjectIds.SetMonitoringModeRequest_Encoding_DefaultBinary: SetMonitoringModeRequest,
    ObjectIds.SetMonitoringModeResponse_Encoding_DefaultBinary: SetMonitoringModeResponse,
    ObjectIds.SetTriggeringRequest_Encoding_DefaultBinary: SetTriggeringRequest,
    ObjectIds.SetTriggeringResponse_Encoding_DefaultBinary: SetTriggeringResponse,
    ObjectIds.DeleteMonitoredItemsRequest_Encoding_DefaultBinary: DeleteMonitoredItemsRequest,
    ObjectIds.DeleteMonitoredItemsResponse_Encoding_DefaultBinary: DeleteMonitoredItemsResponse,
    ObjectIds.CreateSubscriptionRequest_Encoding_DefaultBinary: CreateSubscriptionRequest,
    ObjectIds.CreateSubscriptionResponse_Encoding_DefaultBinary: CreateSubscriptionResponse,
    ObjectIds.ModifySubscriptionRequest_Encoding_DefaultBinary: ModifySubscriptionRequest,
    ObjectIds.ModifySubscriptionResponse_Encoding_DefaultBinary: ModifySubscriptionResponse,
    ObjectIds.SetPublishingModeRequest_Encoding_DefaultBinary: SetPublishingModeRequest,
    ObjectIds.SetPublishingModeResponse_Encoding_DefaultBinary: SetPublishingModeResponse,
    ObjectIds.NotificationMessage_Encoding_DefaultBinary: NotificationMessage,
    ObjectIds.NotificationData_Encoding_DefaultBinary: NotificationData,
    ObjectIds.DataChangeNotification_Encoding_DefaultBinary: DataChangeNotification,
    ObjectIds.MonitoredItemNotification_Encoding_DefaultBinary: MonitoredItemNotification,
    ObjectIds.EventNotificationList_Encoding_DefaultBinary: EventNotificationList,
    ObjectIds.EventFieldList_Encoding_DefaultBinary: EventFieldList,
    ObjectIds.HistoryEventFieldList_Encoding_DefaultBinary: HistoryEventFieldList,
    ObjectIds.StatusChangeNotification_Encoding_DefaultBinary: StatusChangeNotification,
    ObjectIds.SubscriptionAcknowledgement_Encoding_DefaultBinary: SubscriptionAcknowledgement,
    ObjectIds.PublishRequest_Encoding_DefaultBinary: PublishRequest,
    ObjectIds.PublishResponse_Encoding_DefaultBinary: PublishResponse,
    ObjectIds.RepublishRequest_Encoding_DefaultBinary: RepublishRequest,
    ObjectIds.RepublishResponse_Encoding_DefaultBinary: RepublishResponse,
    ObjectIds.TransferResult_Encoding_DefaultBinary: TransferResult,
    ObjectIds.TransferSubscriptionsRequest_Encoding_DefaultBinary: TransferSubscriptionsRequest,
    ObjectIds.TransferSubscriptionsResponse_Encoding_DefaultBinary: TransferSubscriptionsResponse,
    ObjectIds.DeleteSubscriptionsRequest_Encoding_DefaultBinary: DeleteSubscriptionsRequest,
    ObjectIds.DeleteSubscriptionsResponse_Encoding_DefaultBinary: DeleteSubscriptionsResponse,
    ObjectIds.BuildInfo_Encoding_DefaultBinary: BuildInfo,
    ObjectIds.RedundantServerDataType_Encoding_DefaultBinary: RedundantServerDataType,
    ObjectIds.EndpointUrlListDataType_Encoding_DefaultBinary: EndpointUrlListDataType,
    ObjectIds.NetworkGroupDataType_Encoding_DefaultBinary: NetworkGroupDataType,
    ObjectIds.SamplingIntervalDiagnosticsDataType_Encoding_DefaultBinary: SamplingIntervalDiagnosticsDataType,
    ObjectIds.ServerDiagnosticsSummaryDataType_Encoding_DefaultBinary: ServerDiagnosticsSummaryDataType,
    ObjectIds.ServerStatusDataType_Encoding_DefaultBinary: ServerStatusDataType,
    ObjectIds.SessionDiagnosticsDataType_Encoding_DefaultBinary: SessionDiagnosticsDataType,
    ObjectIds.SessionSecurityDiagnosticsDataType_Encoding_DefaultBinary: SessionSecurityDiagnosticsDataType,
    ObjectIds.ServiceCounterDataType_Encoding_DefaultBinary: ServiceCounterDataType,
    ObjectIds.StatusResult_Encoding_DefaultBinary: StatusResult,
    ObjectIds.SubscriptionDiagnosticsDataType_Encoding_DefaultBinary: SubscriptionDiagnosticsDataType,
    ObjectIds.ModelChangeStructureDataType_Encoding_DefaultBinary: ModelChangeStructureDataType,
    ObjectIds.SemanticChangeStructureDataType_Encoding_DefaultBinary: SemanticChangeStructureDataType,
    ObjectIds.Range_Encoding_DefaultBinary: Range,
    ObjectIds.EUInformation_Encoding_DefaultBinary: EUInformation,
    ObjectIds.ComplexNumberType_Encoding_DefaultBinary: ComplexNumberType,
    ObjectIds.DoubleComplexNumberType_Encoding_DefaultBinary: DoubleComplexNumberType,
    ObjectIds.AxisInformation_Encoding_DefaultBinary: AxisInformation,
    ObjectIds.XVType_Encoding_DefaultBinary: XVType,
    ObjectIds.ProgramDiagnosticDataType_Encoding_DefaultBinary: ProgramDiagnosticDataType,
    ObjectIds.Annotation_Encoding_DefaultBinary: Annotation,
}

